require 'fstate.rb'
require 'timer.rb'

# TimerTest.Test()
$tm = Timer.new

module NameGenerator
    LNAMES = [ "Kovacs", "Lakatos", "Szabo", "Horvat", "Lovas", "Gesztes", "Szekely", "Katona", "Karasz", "Hetenyi", "Aradi" ]
    FNAMES_M = [ "Karoly", "Peter", "Geza", "Istvan", "Jozsef", "Artur", "Laszlo", "Gabor", "Szabolcs", "Miklos", "Menyhert", "Elemer", "Balazs", "Vilmos", "Almos" ]
    FNAMES_F = [ "Szilvi", "Katalin", "Jolan", "Maria", "Irma", "Kriszta", "Edina", "Mariann", "Andrea", "Adrienn", "Tunde", "Zsuzsa" ]
    def NameGenerator::GetName(f = true)
        LNAMES[rand(LNAMES.size)] + " " + ( f == true ? FNAMES_M[rand(FNAMES_M.size)] : FNAMES_F[rand(FNAMES_F.size)] )
    end
end


class Munkahely
    attr    :alkalmazottak
    attr :fonok
    attr :titkarno
    
    def initialize
        # @fonok = f      unless f.nil?
        # @titkarno = t   unless  t.nil?
        @alkalmazottak = []
    end

    def fonok=(f)
        @fonok=f
    end

    def titkarno=(t)
        @titkarno=t
    end

    def +(a)
        alkalmazottak << a
        self
    end

    def -(a)
        alkalmazottak.delete(a)
        self
    end
end


class Actor
    def do_it
    end
end



class Dolgozo < Actor
    SHR   = 8      # 8:00 am
    THR   = 17     # 5:00 pm
    # personal identity
    attr    :gender
    attr    :name
    attr    :company
    # states
    attr    :loc_state

    def initialize(mh, k=-1)    
        mh += self
        @company    = mh   unless mh.nil?
    
        if k == -1
            @gender = rand % 1 # (@gender == 0 ? true : false)
        else
            @gender = (k == 0 ? true : false)
        end
        @name = NameGenerator::GetName( @gender == 0 )
    
        s = [ "s", "home", "work", "lunch", "away", "t" ]
        e = { "s" => [ "home", "work"], "home" => "work", "work" => [ "home", "lunch", "away" ], "lunch" => ["work", "home"], "away" => ["work", "home"] }
        @loc_state = FState.new(s, nil, nil, e)
        @loc_state.delegate = self

        # set start state
        if $tm.dtime >= SHR*60 and $tm.dtime < THR*60
            @loc_state.setState("work")
        else
            @loc_state.setState("home")
        end
    end

    def to_s
        @name + ", " + self.class.to_s + "[" + self.id.to_s + "]"
    end

    # location events
    def enter_work
        puts "#{self} at work"
        # @loc_ev = $tm.adde(THR, 0, 0, self)
    end
    
    def enter_home
        puts "#{self} at home!"
        # @loc_ev = $tm.adde(SHR, 0, 0, self)
    end

    def enter_lunch
        lm = 15 + rand(15)
        puts "#{self} is out for lunch for #{lm} minutes!"
        @lunch_end = $tm.dtime + lm
        @lunch_back = @loc_state.last
    end

    def fire(t)
        if t == @loc_ev
            # location event arrived
            if @loc_state.state == "home"
                puts "#{self}: Let's go to work!"
                @loc_state.setState("work")
            else
                puts "#{self}: Go home!"
                @loc_state.setState("home")
            end
        end
    end

    def do_it
        case @loc_state.state
            when "home"
                if $tm.dtime >= SHR*60 and $tm.dtime < THR*60
                    @loc_state.setState("work")
                end
            when "work"
                if $tm.dtime < SHR*60 or $tm.dtime >= THR*60
                    @loc_state.setState("home")
                elsif ($tm.dtime - 12*60).abs < 30
                    @loc_state.setState("lunch")
                end
            when "lunch"
                if $tm.dtime >= @lunch_end
                    @loc_state.back
                end
        end
    end
end

class Fonok < Dolgozo
    def initialize(mh, k=-1)
        super(mh, k)
    end
end

class Alkalmazott < Dolgozo
    def initialize(mh, k=-1)
        super(mh, k)
    end
end


class Titkarno < Alkalmazott
    def initialize(mh, k=-1)
        super(mh, k)
    end
end

class Fejleszto < Alkalmazott
    attr    :work_state

    def initialize(mh, k=-1)
        super(mh, k)

        # 'dolgozik' substate
        s = [ "start", "tervez", "kodol", "support", "debuggol", "kesz" ]
        t = s[5]
        e = { "start" => "tervez", "tervez" => "kodol", "kodol" => ["support", "debuggol", "tervez"], "debuggol" => ["support", "kodol", "kesz"], "support" => ["kodol", "tervez"] }
        @work_state = FState.new(s, nil, t, e)
        @work_state.delegate = self
    end
end

mh1 = Munkahely.new
mh1.fonok = Fonok.new(mh1)
mh1.titkarno = Titkarno.new(mh1, 1)

actors = []

actors << mh1.fonok
actors << mh1.titkarno
actors << Fejleszto.new(mh1)
actors << Fejleszto.new(mh1)

# infinite loop
puts "Time at #{$tm}"
while true
    actors.each{|a| a.do_it}

    # leap timer a half hour
    $tm += 1800
    puts "Time at #{$tm}"

    # wait some ..
    sleep 1
end

